"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.systemStates = void 0;
const model_1 = require("./model");
const controlPanel_1 = __importDefault(require("./controlPanel"));
const { heater, pump, valve, thermometer, logMessage } = model_1.model;
exports.systemStates = {
    notReady: 'Not ready',
    ready: 'Ready',
    starting: 'Starting',
    running: 'Running',
    stopping: 'Stopping'
};
let timer;
const isSystemReady = () => {
    return heater.isReady() && pump.isReady() && valve.isReady() && thermometer.isReady();
};
const shouldStartHeating = () => {
    return controlPanel_1.default.systemState.value === exports.systemStates.ready
        && controlPanel_1.default.mainSwitch.value
        && controlPanel_1.default.setpoint.value !== undefined
        && thermometer.temperature !== undefined
        && controlPanel_1.default.setpoint.value > thermometer.temperature;
};
const stopHeatingIfNecessary = (valvePosition) => {
    if (valvePosition === 0) {
        if (timer === undefined) {
            timer = setTimeout(() => {
                stopHeating();
            }, 5000);
        }
    }
    else {
        if (timer !== undefined) {
            clearTimeout(timer);
            timer = undefined;
        }
    }
};
const adjustValve = () => {
    // @ts-ignore
    let valvePosition = 100 * (controlPanel_1.default.setpoint.value - thermometer.temperature + 1) / 3;
    if (valvePosition > 100) {
        valvePosition = 100;
    }
    else if (valvePosition < 0) {
        valvePosition = 0;
    }
    valve.setPosition(valvePosition);
    stopHeatingIfNecessary(valvePosition);
};
const startHeating = () => {
    if (shouldStartHeating()) {
        logMessage('Heating activated');
        if (timer) {
            clearTimeout(timer);
            timer = undefined;
        }
        controlPanel_1.default.systemState.setValue(exports.systemStates.starting);
        pump.switchOn();
        timer = setTimeout(() => {
            adjustValve();
            timer = setTimeout(() => {
                timer = undefined;
                heater.switchOn();
                controlPanel_1.default.systemState.setValue(exports.systemStates.running);
                adjustValve();
                if (!controlPanel_1.default.mainSwitch.value) {
                    stopHeating();
                }
            }, 5000);
        }, 5000);
    }
};
const stopHeating = () => {
    if (controlPanel_1.default.systemState.value === exports.systemStates.running) {
        logMessage('Heating deactivated');
        if (timer) {
            clearTimeout(timer);
            timer = undefined;
        }
        controlPanel_1.default.systemState.setValue(exports.systemStates.stopping);
        heater.switchOff();
        timer = setTimeout(() => {
            valve.setPosition(0);
            timer = setTimeout(() => {
                timer = undefined;
                pump.switchOff();
                controlPanel_1.default.systemState.setValue(exports.systemStates.ready);
                if (shouldStartHeating()) {
                    startHeating();
                }
            }, 5000);
        }, 5000);
    }
};
const emergencyStop = () => {
    if (timer) {
        clearTimeout(timer);
        timer = undefined;
    }
    controlPanel_1.default.systemState.setValue(exports.systemStates.notReady);
    heater.switchOff();
    pump.switchOff();
    valve.setPosition(0);
};
const onModelChange = () => {
    if (isSystemReady()) {
        controlPanel_1.default.systemState.setValue(exports.systemStates.ready);
        if (controlPanel_1.default.mainSwitch.value) {
            startHeating();
        }
    }
    else {
        switch (controlPanel_1.default.systemState.value) {
            case exports.systemStates.notReady:
                break;
            case exports.systemStates.ready:
                controlPanel_1.default.systemState.setValue(exports.systemStates.notReady);
                break;
            default:
                emergencyStop();
        }
    }
};
const onTemperatureChange = () => {
    if (controlPanel_1.default.systemState.value === exports.systemStates.running) {
        adjustValve();
    }
    else if (shouldStartHeating()) {
        startHeating();
    }
};
exports.default = {
    startHeating,
    stopHeating,
    isSystemReady,
    onTemperatureChange,
    onModelChange
};
