import { model } from "./model";
import controlPanel from "./controlPanel";

const { heater, pump, valve, thermometer, logMessage } = model;

export const systemStates = {
    notReady: 'Not ready',
    ready: 'Ready',
    starting: 'Starting',
    running: 'Running',
    stopping: 'Stopping'
}

let timer: ReturnType<typeof setTimeout> | undefined;

const isSystemReady = () => {
    return heater.isReady() && pump.isReady() && valve.isReady() && thermometer.isReady();
}

const shouldStartHeating = () => {
    return controlPanel.systemState.value === systemStates.ready
        && controlPanel.mainSwitch.value
        && controlPanel.setpoint.value !== undefined
        && thermometer.temperature !== undefined
        && controlPanel.setpoint.value > thermometer.temperature;
}

const stopHeatingIfNecessary = (valvePosition: number) => {
    if (valvePosition === 0) {
        if (timer === undefined) {
            timer = setTimeout(() => {
                stopHeating();
            }, 5000);
        }
    } else {
        if (timer !== undefined) {
            clearTimeout(timer);
            timer = undefined;
        }
    }
}

const adjustValve = () => {
    // @ts-ignore
    let valvePosition = 100 * (controlPanel.setpoint.value - thermometer.temperature + 1) / 3;
    if (valvePosition > 100) {
        valvePosition = 100;
    } else if (valvePosition < 0) {
        valvePosition = 0;
    }
    valve.setPosition(valvePosition);
    stopHeatingIfNecessary(valvePosition);
}

const startHeating = () => {
    if (shouldStartHeating()) {
        logMessage('Heating activated');
        if (timer) {
            clearTimeout(timer);
            timer = undefined;
        }
        controlPanel.systemState.setValue(systemStates.starting);
        pump.switchOn();
        timer = setTimeout(() => {
            adjustValve();
            timer = setTimeout(() => {
                timer = undefined;
                heater.switchOn();
                controlPanel.systemState.setValue(systemStates.running);
                adjustValve();
                if (!controlPanel.mainSwitch.value) {
                    stopHeating();
                }
            }, 5000);
        }, 5000);
    }
}

const stopHeating = () => {
    if (controlPanel.systemState.value === systemStates.running) {
        logMessage('Heating deactivated');
        if (timer) {
            clearTimeout(timer);
            timer = undefined;
        }
        controlPanel.systemState.setValue(systemStates.stopping);
        heater.switchOff();
        timer = setTimeout(() => {
            valve.setPosition(0);
            timer = setTimeout(() => {
                timer = undefined;
                pump.switchOff();
                controlPanel.systemState.setValue(systemStates.ready);
                if (shouldStartHeating()) {
                    startHeating();
                }
            }, 5000);
        }, 5000);
    }
}

const emergencyStop = () => {
    if (timer) {
        clearTimeout(timer);
        timer = undefined;
    }
    controlPanel.systemState.setValue(systemStates.notReady);
    heater.switchOff();
    pump.switchOff();
    valve.setPosition(0);
}

const onModelChange = () => {
    if (isSystemReady()) {
        controlPanel.systemState.setValue(systemStates.ready);
        if (controlPanel.mainSwitch.value) {
            startHeating();
        }
    } else {
        switch (controlPanel.systemState.value) {
            case systemStates.notReady:
                break;
            case systemStates.ready:
                controlPanel.systemState.setValue(systemStates.notReady);
                break;
            default:
                emergencyStop();
        }
    }
}

const onTemperatureChange = () => {
    if (controlPanel.systemState.value === systemStates.running) {
        adjustValve();
    } else if (shouldStartHeating()) {
        startHeating();
    }
}

export default {
    startHeating,
    stopHeating,
    isSystemReady,
    onTemperatureChange,
    onModelChange
}