import { getSystemModel } from "@stargate-system/controller";
import { DeviceSubscription, DeviceModel, DeviceState } from "@stargate-system/model";
import { GateBoolean, GateNumber } from "@stargate-system/core"

let heaterSwitch: GateBoolean | undefined;
let pumpSwitch: GateBoolean | undefined;
let valvePosition: GateNumber | undefined;

const pump = {
    switchOn: () => pumpSwitch?.setValue(true),
    switchOff: () => pumpSwitch?.setValue(false),
    isReady: () => pumpSwitch !== undefined
}

const heater = {
    switchOn: () => heaterSwitch?.setValue(true),
    switchOff: () => heaterSwitch?.setValue(false),
    isReady: () => heaterSwitch !== undefined
}

const valve = {
    setPosition: (value: number) => valvePosition?.setValue(value),
    isReady: () => valvePosition !== undefined
}

const thermometer = {
    temperature: undefined,
    onTemperatureChange: () => { },
    isReady: () => thermometer.temperature !== undefined
}

const logMessage = (message: string) => {
    systemModel.serverStorage.append(
        'heating.txt',
        `${(new Date()).toLocaleString()} > ${message}\n`,
        './logs'
    )
}

export const model = {
    heater,
    pump,
    valve,
    thermometer,
    onModelChange: () => { },
    logMessage
}

const systemModel = getSystemModel();

const initDeviceModel = (deviceModel: DeviceModel | undefined, onDeviceUp: () => void, onDeviceDown: () => void) => {
    if (deviceModel && deviceModel.state.value === DeviceState.up) {
        onDeviceUp();
        model.onModelChange();
        deviceModel.state.subscribe(() => {
            if (deviceModel.state.value !== DeviceState.up) {
                onDeviceDown();
                model.onModelChange();
            }
        });
    } else {
        onDeviceDown();
        model.onModelChange();
    }
}

new DeviceSubscription(
    systemModel,
    (device) => device.name.value === 'Heater',
    (device) => initDeviceModel(
        device,
        () => heaterSwitch = device?.gateValues.values[0]?.gateValue as GateBoolean | undefined,
        () => heaterSwitch = undefined
    )
);

new DeviceSubscription(
    systemModel,
    (device) => device.manifest.uuid === '0be8e7f0-2c70-4cfd-a10e-36ba70222f3c',
    (device) => initDeviceModel(
        device,
        () => pumpSwitch = device?.gateValues.values[0]?.gateValue as GateBoolean | undefined,
        () => pumpSwitch = undefined
    )
);

new DeviceSubscription(
    systemModel,
    (device) => device.info.value === 'valve',
    (device) => initDeviceModel(
        device,
        () => valvePosition = device?.gateValues.values[0]?.gateValue as GateNumber | undefined,
        () => valvePosition = undefined
    )
);

new DeviceSubscription(
    systemModel,
    (device) => (device.group.value === 'Living Room') && (device.name.value === 'Weather Station'),
    (device) => initDeviceModel(
        device,
        () => {
            const temperature = device?.gateValues.find((value) => value.gateValue.valueName === 'Temperature');
            if (temperature) {
                thermometer.temperature = temperature.modelValue.value;
                thermometer.onTemperatureChange();
                temperature.modelValue.subscribe(() => {
                    thermometer.temperature = temperature.modelValue.value;
                    thermometer.onTemperatureChange();
                });
            } else {
                thermometer.temperature = undefined;
            }
        },
        () => thermometer.temperature = undefined
    )
);