import { GateDevice, Directions } from "@stargate-system/device";
import { getSystemModel } from "@stargate-system/controller";
import { DeviceSubscription, DeviceModel, DeviceState } from "@stargate-system/model";

GateDevice.setName('Weather Station');

const temperature = GateDevice.factory.createFloat(Directions.output);
temperature.valueName = 'Temperature';

const humidity = GateDevice.factory.createInteger(Directions.output);
humidity.valueName = 'Humidity (%)';
humidity.setRange([0, 100]);

GateDevice.start();

const initDevice = (device: DeviceModel | undefined) => {
    if (device) {
        device.gateValues.values[0].modelValue.subscribe(() => { });
    }
}

const systemModel = getSystemModel();

const model = {
    temperature: 14,
    externalTemperature: 14
}

const heater = new DeviceSubscription(systemModel, (device) => device.name.value === 'Heater', initDevice);
const pump = new DeviceSubscription(systemModel, (device) => device.manifest.uuid === '0be8e7f0-2c70-4cfd-a10e-36ba70222f3c', initDevice);
const valve = new DeviceSubscription(systemModel, (device) => device.info.value === 'valve', initDevice);

const isHeaterActive = () => {
    return (heater.deviceModel?.state.value === DeviceState.up) && (heater.deviceModel?.gateValues.values[0].gateValue.value);
}

const isPumpActive = () => {
    return (pump.deviceModel?.state.value === DeviceState.up) && (pump.deviceModel?.gateValues.values[0].gateValue.value);
}

const getValveState = () => {
    if (valve.deviceModel?.state.value !== DeviceState.up) {
        return 0;
    }
    return valve.deviceModel.gateValues.values[0].gateValue.value;
}

const recalculate = () => {
    calculateTemperature();
    calculateHumidity();
}

const calculateTemperature = () => {
    let heatIn = 0;
    const valveState = getValveState();
    if (isHeaterActive() && isPumpActive() && (valveState > 0)) {
        heatIn = valveState / 200;
    }
    const heatOut = (model.temperature - model.externalTemperature) / 40;
    model.temperature += heatIn - heatOut;
    temperature.setValue(Math.round(model.temperature * 10) / 10);
}

const calculateHumidity = () => {
    let calculated = 82 - (model.temperature - model.externalTemperature) * 3;
    if (calculated > 100) {
        calculated = 100;
    } else if (calculated < 0) {
        calculated = 0;
    }
    humidity.setValue(Math.round(calculated));
}

setInterval(recalculate, 500);
