"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Device = void 0;
const core_1 = require("@stargate-system/core");
const ControllerContext_1 = __importDefault(require("../controller/ControllerContext"));
const DeviceContext_1 = __importDefault(require("../device/DeviceContext"));
const ServerStorageRequestListener_1 = require("../common/ServerStorageRequestListener");
class Device {
    constructor(connector) {
        this._subscriptions = new Map();
        this.sendValue = (value) => {
            if (this._connection.state === core_1.ConnectionState.ready) {
                this._connection.sendValue(value);
            }
        };
        this.subscribe = (subscribedValueId, consumer) => {
            const subscribers = this._subscriptions.get(subscribedValueId);
            if (subscribers) {
                this._subscriptionBuffer.subscribe(subscribedValueId);
                subscribers.set(consumer.id, consumer);
            }
        };
        this.unsubscribe = (unsubscribedValueId, consumerId) => {
            const subscribers = this._subscriptions.get(unsubscribedValueId);
            if (subscribers) {
                subscribers.delete(consumerId);
                if (subscribers.size === 0) {
                    this._subscriptionBuffer.unsubscribe(unsubscribedValueId);
                }
            }
        };
        this.unsubscribeConsumer = (consumerId) => {
            this._subscriptions.forEach((map, valueId) => {
                if (map.has(consumerId)) {
                    map.delete(consumerId);
                    if (map.size === 0) {
                        this._subscriptionBuffer.unsubscribe(valueId);
                    }
                }
            });
        };
        this._register = () => {
            try {
                DeviceContext_1.default.deviceRegistry.add(this, this._id);
                this._connection.functionalHandler.sendCommand(core_1.Keywords.ready);
                this._connection.setReady();
                DeviceContext_1.default.notifyPipes(core_1.EventName.deviceConnected, this);
                ControllerContext_1.default.forwardDeviceEvent(core_1.EventName.deviceConnected, [JSON.stringify(this._manifest)]);
                console.log("Connected device: " + this._id);
            }
            catch (err) {
                this._connection.close();
                console.log('Failed registering device', err);
            }
        };
        this._routeDeviceMessage = (valueMessage) => {
            const receiversMap = new Map;
            valueMessage.forEach((change) => {
                const [valueId, message] = change;
                const idWithParent = core_1.AddressMapper.appendParentId(this._id, valueId);
                const receivers = this._subscriptions.get(valueId);
                if (receivers) {
                    receivers.forEach((consumer, consumerId) => {
                        let receiver = receiversMap.get(consumerId);
                        if (receiver) {
                            receiver[1].push([idWithParent, message]);
                        }
                        else {
                            receiver = [consumer, [[idWithParent, message]]];
                            receiversMap.set(consumerId, receiver);
                        }
                    });
                }
            });
            receiversMap.forEach((value) => {
                value[0].sendValueMessage(value[1]);
            });
        };
        if (!connector.id || !connector.manifest) {
            connector.connection.close();
            throw new Error('On creating device: connector has incomplete data');
        }
        this._id = connector.id;
        this._manifest = connector.manifest;
        this._connection = connector.connection;
        this._connection.onValueMessage = this._routeDeviceMessage;
        (0, ServerStorageRequestListener_1.setServerStorageRequestListeners)(this._connection, this._id);
        this._subscriptionBuffer = new core_1.SubscriptionBuffer((subscribed) => this._connection.functionalHandler.sendCommand(core_1.Keywords.subscribe, subscribed), (unsubscribed) => this._connection.functionalHandler.sendCommand(core_1.Keywords.unsubscribe, unsubscribed));
        this._manifest.values.forEach((value) => {
            this._subscriptions.set(value.id, new Map);
        });
        this._connection.addStateChangeListener((state) => {
            if (state === core_1.ConnectionState.closed) {
                DeviceContext_1.default.deviceRegistry.remove(this._id);
                DeviceContext_1.default.notifyPipes(core_1.EventName.deviceDisconnected, this._id);
                ControllerContext_1.default.forwardDeviceEvent(core_1.EventName.deviceDisconnected, [this._id]);
                console.log("Disconnected device: " + this._id);
            }
        });
        const connectedDevice = DeviceContext_1.default.deviceRegistry.getByKey(this._id);
        if (connectedDevice) {
            console.log("Closed existing connection with " + connectedDevice.id);
            connectedDevice._connection.addStateChangeListener((state) => {
                if (state === core_1.ConnectionState.closed) {
                    setTimeout(this._register, 0);
                }
            });
            connectedDevice._connection.close();
        }
        else {
            this._register();
        }
    }
    get id() {
        return this._id;
    }
    get manifest() {
        return this._manifest;
    }
}
exports.Device = Device;
